
import unittest
from isharp.core import DataBroker,MatrixHeader,Matrix,AcquireContentReturnValue,Revision
from unittest.mock import MagicMock
from unittest.mock import patch
from isharp.core import StorageMethod
from datetime import datetime
from isharp.core import RevisionInfo
from isharp.core import AbstractDataBroker

test_header = MatrixHeader(name="hello", revision_id=5, storage_method="test", path=".", memory_style=None,
                           description="None")
test_content = {}
time_now = datetime.now()

test_revision = Revision("abc",RevisionInfo("who","what",time_now))

class TestAbstractBroker(unittest.TestCase):
    @patch.multiple(StorageMethod, __abstractmethods__=set())
    def setUp(self):
        self.mock_storage_method = StorageMethod("test")
        self.broker = AbstractDataBroker(self.mock_storage_method)

    def tearDown(self):
        pass

    def test_invalid_protocol(self):
        with self.assertRaises(DataBroker.ProtocolException):
            self.broker.checkout("xyz://hello?myParam=1")

    def test_checkout_file_already_checked_out(self):
        self.mock_acquire_content_result()
        self.broker.checkout("test:///file_name_1.csv?format=CSV")
        with self.assertRaises(DataBroker.CheckoutException):
            self.broker.checkout("test:///file_name_1.csv?format=CSV")

    def test_commit(self):
        self.mock_acquire_content_result()
        self.mock_commit_result()
        testurl = "test:///file_name_1.csv?format=CSV"
        m = self.broker.checkout(testurl)
        revision_info = RevisionInfo("who","what",time_now)
        revision = self.broker.commit(m,revision_info)
        self.assertEqual(revision_info,revision.revision_info)

    def test_release(self):
        self.mock_acquire_content_result()
        self.mock_commit_result()
        testurl = "test:///file_name_1.csv?format=CSV"
        m = self.broker.checkout(testurl)
        self.broker.release(m)
        with self.assertRaises(DataBroker.CheckoutException):
            revision = self.broker.commit(m, test_revision.revision_info)


    def test_get_simple_matrix(self):
        self.mock_acquire_content_result()
        testurl = "test:///file_name_1.csv?format=CSV"
        m = self.broker.checkout(testurl)
        self.assertEqual(test_header, m.matrix_header,"header should be test header")
        self.assertEqual(test_content, m.content,"Content should be empty hashtable")
        self.assertEqual(testurl,m.url.url, "URL should be test url")
        self.mock_storage_method.acquireContent.assert_called_once_with(params={"format": "CSV"}, path="/file_name_1.csv",version_id =None)

    def test_list(self):
        self.mock_storage_method.list = MagicMock(return_value=[test_header])
        self.assertEqual([test_header],self.broker.list(),"Sould return single header")
        self.mock_storage_method.list.assert_called_once()

    def mock_commit_result(self):
        self.mock_storage_method.storeContent = MagicMock(return_value=test_revision)

    def mock_acquire_content_result(self):
        self.mock_storage_method.acquireContent = \
            MagicMock(return_value=(AcquireContentReturnValue(header=test_header,content=test_content)))



import unittest
import test.testutil.mongo_arctic_utils as marc
import test.testutil.log_utils as lu
import test.testutil.pandas_utils as pu
from arctic import Arctic
import pandas as pd
import numpy as np
import datetime
from isharp.arctic_broker.broker_impl.arctic_storage_method import  ArcticStorageMethod
from isharp.core import StorageMethod

class TestMongoBroker(unittest.TestCase):
    def setUp(self):
        self.arctic = Arctic('localhost')
        self.library_name = marc.random_library(self.arctic)
        self.symbol_name = 'symbol'
        lu.logger.info("Created test library {}".format(self.library_name))
        self.arctic.initialize_library(self.library_name)
        pd = pu.create_simple_series(['a','b','c'] , 50)
        self.arctic[self.library_name].write(self.symbol_name,pd)

    def tearDown(self):
        self.arctic.delete_library(self.library_name)
        lu.logger.info("deleted test library {}".format(self.library_name))

    def test_acquire_data(self):
        method = ArcticStorageMethod(self.arctic)
        result  = method.acquireContent("{}/symbol".format(self.library_name),{})
        data_frame = result.content
        self.assertIs(3,len(data_frame.columns))
        self.assertIsNone(result.header.description)

    def test_acquire_missing_data(self):
        method = ArcticStorageMethod(self.arctic)
        with self.assertRaisesRegexp(StorageMethod.ResourceException,'^ticker nosymbol not found$'):
            method.acquireContent("{}/nosymbol".format(self.library_name),{})
        with self.assertRaisesRegexp(StorageMethod.ResourceException,'^library nolib not found$'):
            method.acquireContent("nolib/nosymbol".format(self.library_name),{})













import datetime
import pandas as pd
import numpy as np
import tempfile
import os
def create_simple_series(columns,number_days):

    data = np.array([np.arange(number_days)] * len(columns)).T
    todays_date = datetime.datetime.now().date()
    index = pd.date_range(todays_date - datetime.timedelta(number_days), periods=number_days, freq='D')
    return pd.DataFrame(data,index=index,columns=columns)

def testit():
    columns = ['A','B','C']
    data = np.array([np.arange(10)] * 3).T
    todays_date = datetime.datetime.now().date()
    index = pd.date_range(todays_date - datetime.timedelta(10), periods=10, freq='D')
    df  = pd.DataFrame(data,index=index,columns=columns)

    data_path = tempfile.mkdtemp(prefix="util_test_")
    print("created temp dir {}".format(data_path))
    csvpath = os.path.join(data_path,"test.csv")
    df.to_csv(csvpath)

    df = pd.DataFrame.from_csv(csvpath)



if __name__ == '__main__':
    testit()




import unittest
import test.testutil.mongo_arctic_utils as marc
import test.testutil.log_utils as lu
import test.testutil.pandas_utils as pu
from isharp.arctic_broker.broker_impl.arctic_data_broker import ArcticBroker
from arctic import Arctic
import pandas as pd
import numpy as np
import datetime
from isharp.core import RevisionInfo



class TestArcticBroker(unittest.TestCase):

    def setUp(self):
        self.arctic = Arctic('localhost')
        self.library_name = marc.random_library(self.arctic)
        lu.logger.info("Created test library {}".format(self.library_name))
        self.arctic.initialize_library(self.library_name)
        simple_pd =pu.create_simple_series(['a','b','c'],5)
        lib = self.arctic[self.library_name]
        lib.write("symbol",simple_pd)


    def test_checkout_and_checkin_arctic(self):
        url = "arctic:///{}/symbol".format(self.library_name)
        broker = ArcticBroker(self.arctic)
        matrix = broker.checkout(url)
        num_rows_original_version_1 = len(matrix.content.index)
        self.assertEquals("1", matrix.matrix_header.revision_id)
        df  = matrix.content.append(pd.DataFrame(data=np.random.randn(1, len(matrix.content.columns)), index=[matrix.content.index[-1] + datetime.timedelta(days=1)],columns=matrix.content.columns))

        revision_info = RevisionInfo(who="Jeremy Ward", what="first test commit", when=datetime.datetime(year=2000,month=1,day=13))
        broker.commit(matrix.replace_content(df),revision_info)
        matrix = broker.checkout(url)
        self.assertEquals("2", matrix.matrix_header.revision_id)
        num_rows_original_version_2 = len(matrix.content.index)
        self.assertEquals(1, num_rows_original_version_2 - num_rows_original_version_1)
        broker.release(matrix)
        matrix = broker.checkout(url, version="1")
        self.assertEquals(num_rows_original_version_1,len(matrix.content.index))
        broker.release(matrix)

        matrix = broker.checkout(url, version="2")
        self.assertEquals(num_rows_original_version_2,len(matrix.content.index))


    def test_list(self):
        broker = ArcticBroker(self.arctic)
        result = broker.list()
        self.assertEquals(1, len(result))



    def tearDown(self):
        self.arctic.delete_library(self.library_name)
        lu.logger.info("deleted test library {}".format(self.library_name))











from unittest import TestCase
from unittest.mock import MagicMock,patch,Mock
from isharp.core import DataBroker,CombiBroker, MatrixUrl, MatrixHeader,Matrix

class TestCombiBroker(TestCase):

    @patch.multiple(DataBroker, __abstractmethods__=set())
    def setUp(self):
        self.foo_broker = Mock()
        self.bar_broker = Mock()
        self.foo_broker.list = MagicMock(return_value=[MatrixHeader(name="foo",revision_id=None,storage_method=None,path=None,memory_style=None,description=None)])
        self.bar_broker.list = MagicMock(return_value = [MatrixHeader(name="bar",revision_id=None,storage_method=None,path=None,memory_style=None,description=None)])
        self.combi_broker = CombiBroker(
            {
                "foo": self.foo_broker,
                "bar": self.bar_broker
            }
        )

    def test_checkout_with_invalid_protocol(self):
        with self.assertRaises(DataBroker.ProtocolException):
            self.combi_broker.checkout("test:///file_name_1.csv?format=CSV")

    def test_checkout_with_valid_protocol(self):
        self.combi_broker.checkout("foo://abc.com/xyz")
        self.bar_broker.checkout.assert_not_called()
        self.foo_broker.checkout.assert_called_with("foo://abc.com/xyz",None)

    def test_checkin_with_valid_protocol(self):
        m = Matrix(matrix_header=None,content=None, url = MatrixUrl("foo://abc.com/xyz"))
        ri=None
        self.combi_broker.commit(m,ri)
        self.bar_broker.commit.assert_not_called()
        self.foo_broker.commit.assert_called_with(m,ri)
        self.foo_broker.checkout.assert_not_called()
        self.bar_broker.checkout.assert_not_called()
        self.foo_broker.checkout.assert_not_called()

    def test_checkin_with_invalid_protocol(self):
        with self.assertRaises(DataBroker.ProtocolException):
            m = Matrix(matrix_header=None, content=None, url=MatrixUrl("test://abc.com/xyz"))
            ri = None
            self.combi_broker.commit(m, ri)

    def test_list(self):
        ret = self.combi_broker.list()
        self.assertEquals(2, len(ret))
        self.foo_broker.list.assert_called()
        self.bar_broker.list.assert_called()
        self.bar_broker.checkout.assert_not_called()
        self.foo_broker.checkout.assert_not_called()













""" Service unit testing best practice.
"""
from nameko.testing.services import worker_factory
from isharp.broker_service.server import DataBrokerService
import pandas as pd
from isharp.core import Matrix
from unittest import TestCase

class TestBrokerService(TestCase):

    def test_broker_service(self):
        broker_service = worker_factory(DataBrokerService)
        broker_service.data_broker_rpc.checkout.side_effect = lambda x: Matrix(None,None,None)
        result = broker_service.data_broker_rpc.checkout("randomRubbish")
        self.assertIsNone(result.matrix_header)


from typing import List
from isharp.core import DataBroker, MatrixHeader, Matrix, AcquireContentReturnValue, Revision
from isharp.core import MatrixUrl
from isharp.core import RevisionInfo
from urllib.parse import urlparse
from nameko.standalone.rpc import ClusterRpcProxy
import logging
logger = logging.getLogger(__name__)


def remote_config(net_location:str):
    return {
        'serializer': 'pickle',
        'AMQP_URI': 'pyamqp://guest:guest@{}'.format(net_location),
        'rpc_exchange': 'nameko-rpc',
        'max_workers': 10,
        'parent_calls_tracked': 10
    }

class PooledBrokerConnection(DataBroker):
    def __init__(self, net_location: str):
        logger.info("creating remote client at {}".format(net_location) )
        conf = remote_config(net_location)
        self.rpc_proxy = ClusterRpcProxy(conf)
        self.proxy = self.rpc_proxy.start()
        self.net_location = net_location

    def stop(self):
        logger.info("closing remote client at {}".format(self.net_location))
        self.rpc_proxy.stop()


    def commit(self, matrix: Matrix, revisionInfo: RevisionInfo) -> Revision:
        pass

    def list(self) -> List[MatrixHeader]:
        return self.proxy.data_broker_service.list()

    def checkout(self, url: str, version_id=None) -> Matrix:
        return self.proxy.data_broker_service.checkout(url)


class BrokerConnectionPool(DataBroker):
    def __init__(self):
        self.pool = {}

    def __enter__(self):
        return self

    def  __exit__(self, exc_type, exc_value, traceback):
        for thisConnection in self.pool.values():
            thisConnection.stop()

    def _connect(self, url: str):
        connection_key = self._conn_details(url)
        if connection_key not in self.pool.keys():
            self.pool[connection_key] = PooledBrokerConnection(connection_key)
        return self.pool[connection_key]

    def _conn_details(self, url: str) -> str:
        url_components = urlparse(url)
        return  url_components.netloc

    def checkout(self, url: str, version_id=None) -> Matrix:
        return self._connect(url).checkout(url, version_id)

    def commit(self, matrix: Matrix, revisionInfo: RevisionInfo) -> Revision:
        return self._connect(matrix.url.url).commit(matrix, revisionInfo)

    def list(self) -> List[MatrixHeader]:
        raise NotImplementedError

    def list(self,network_location):
        return self._connect(network_location).list()







from .file_storage_method import FileStorageMethod
from isharp.core import AbstractDataBroker
import logging

class SimpleFileBroker(AbstractDataBroker):
    def __init__(self,root_directory):
        super().__init__(FileStorageMethod(root_directory))








from nameko.extensions import DependencyProvider
from nameko.rpc import rpc, RpcProxy
from isharp.core import  MatrixHeader,CombiBroker
from typing import List
from isharp.core import  DataBroker,Matrix,RevisionInfo,Revision
import logging

logger = logging.getLogger(__name__)

class DataBrokerDelegate(DependencyProvider):

    def get_dependency(self, worker_ctx):
        print("getting data broker delegates")
        data_brokers =worker_ctx.container.config.get('data_brokers')
        print("got {} data brokers".format(len(data_brokers)))
        return  CombiBroker(data_brokers)

class DataBrokerService(DataBroker):
    name="data_broker_service"
    data_broker_rpc = RpcProxy("data_broker")
    delegate = DataBrokerDelegate()

    @rpc
    def checkout(self, url: str, version_id=None) -> Matrix:
        ret_val = self.delegate.checkout(url,version_id)
        logger.debug("call to delegate successful")
        return ret_val
    @rpc
    def commit(self, matrix: Matrix, revisionInfo: RevisionInfo) -> Revision:
        return super().commit(matrix, revisionInfo)
    @rpc
    def release(self, matrix) -> None:
        pass
    @rpc
    def list(self) -> List[MatrixHeader]:
        pass








from enum import Enum
from typing import List
import dataclasses
from urllib.parse import urlparse
from urllib.parse import parse_qsl
import datetime
import abc


import logging
logger = logging.getLogger(__name__)



from collections import namedtuple
class MemStyles(Enum):
    DATA_FRAME = 1
    TREE = 2




class MatrixUrl:
    def __init__(self,url):
        self.url = url
        self.url_components = urlparse(url)
        logger.debug("Parsed Url: host:[{}] path:[{}] port:[{}] scheme:[{}]".format(self.host(),self.path(),self.port(),self.scheme()))

    def params(self):
        params = parse_qsl(self.url_components.query)
        ret_val = {}
        for this_param in params:
            ret_val[this_param[0]] = this_param[1]
        return ret_val

    def host(self):
        return self.url_components.hostname

    def path(self):
        return  self.url_components.path

    def port(self):
        return self.url_components.port

    def scheme(self):
        return self.url_components.scheme


@dataclasses.dataclass(frozen=True)
class MatrixHeader:
    name: str
    revision_id: str
    storage_method: str
    path: str
    memory_style:MemStyles
    description: str

@dataclasses.dataclass(frozen=True)
class Matrix:
    matrix_header: MatrixHeader
    content: object
    url: MatrixUrl

    def replace_content(self,new_content):
        return Matrix(matrix_header=self.matrix_header,url=self.url,content=new_content)


@dataclasses.dataclass(frozen=True)
class RevisionInfo:
    who: str
    what: str
    when: datetime.datetime


@dataclasses.dataclass(frozen=True)
class Revision:
    id: str
    revision_info: RevisionInfo


class DataBroker(abc.ABC):

    class CheckoutException(Exception):
        pass

    class ProtocolException(Exception):
        pass

    @abc.abstractmethod
    def checkout(self, url:str, version_id=None)->Matrix:
        pass

    @abc.abstractmethod
    def commit(self,matrix:Matrix,revisionInfo:RevisionInfo)->Revision:
        pass

    def release(self,matrix)->None:
        pass

    @abc.abstractmethod
    def list(self)->List[MatrixHeader]:
        pass



AcquireContentReturnValue = namedtuple('AcquireContentReturnValue', 'header content')


class StorageMethod(abc.ABC):
    class ParameterException(Exception):
        pass
    class ResourceException(Exception):
        pass

    def __init__(self, name, required_params={}):
        self.required_parameters = required_params
        self.name = name

    @abc.abstractmethod
    def acquireContent(self,path,params,version_id=None)->AcquireContentReturnValue:
        self._check_params(params)

    @abc.abstractmethod
    def storeContent(self,path,params,content,revision_info)->Revision:
        self._check_params(params)


    @abc.abstractmethod
    def list(self)->List[MatrixHeader]:
        pass
    def _check_params(self,params):
        for required_param in self.required_parameters:
            if not required_param in params:
                raise self.ParameterException("format parameter missing or unset")


class AbstractDataBroker(DataBroker):
    def __init__(self,storage_method):
        super().__init__()
        self.storage_method = storage_method
        self.register = []

    def _assert_not_checked_out(self,matrix_url):
        if matrix_url.path() in self.register:
            raise DataBroker.CheckoutException("matrix [{}] is already checked out".format(matrix_url.path()))

    def _assert_checked_out(self,matrix_url):
        if matrix_url.path() not in self.register:
            raise DataBroker.CheckoutException("matrix [{}] is not already checked out".format(matrix_url.path()))

    def checkout(self, matrix_url_str,version=None):
        logger.debug("Abstract  broker called with {}".format(matrix_url_str))
        matrix_url = MatrixUrl(matrix_url_str)
        if matrix_url.scheme() == self.storage_method.name:
            self._assert_not_checked_out(matrix_url)
            self.register.append(matrix_url.path())
            checkout_result = self.storage_method.acquireContent(path=matrix_url.path(), params=matrix_url.params(),version_id=version)
            ret_val =Matrix(checkout_result.header,checkout_result.content,matrix_url)
            logger.debug("Abstract data broker about to return matrix")
            return ret_val
        else:
            raise DataBroker.ProtocolException()

    def commit(self, matrix, revisionInfo):
        self._assert_checked_out(matrix.url)
        revision = self.storage_method.storeContent(path=matrix.matrix_header.path,content=matrix.content,revision_info=revisionInfo,params=matrix.url.params())
        self.register.remove(matrix.url.path())
        return revision

    def release(self, matrix):
        self.register.remove(matrix.url.path())

    def list(self):
        return self.storage_method.list()





class CombiBroker(DataBroker):
    def __init__(self,registry):
        print('Creating combi broker with keys {}'.format(registry.keys()))
        self.registry = registry

    def checkout(self, url, version_id=None) -> Matrix:
        logger.info("combi broker called with {}".format(url))
        parsed_url = MatrixUrl(url)
        return self._delegate(parsed_url.scheme()).checkout(url,version_id)

    def _delegate(self,scheme)->DataBroker:
        if (scheme in self.registry):
            return self.registry[scheme]
        else:
            raise DataBroker.ProtocolException("invalid protocol")


    def commit(self, matrix: Matrix, revisionInfo: RevisionInfo) -> Revision:
        return self._delegate(matrix.url.scheme()).commit(matrix,revisionInfo)

    def list(self) -> List[MatrixHeader]:
        logger.info("combi broker list method called")
        ret_val = []
        for this_delegate in self.registry.values():
            logger.info("getting listing from delegate")
            ret_val.append(this_delegate.list())
        logger.info("combi broker returning listing with {} values".format(len(ret_val)))
        return ret_val






#!/usr/local/bin/python
# -*- coding: utf-8 -*-
import re
import sys
import isharp.yaml_support  as iYaml
import nameko.cli.main
if __name__ == '__main__':
    sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0])
    iYaml.set_up_unsafe_loader()
    sys.exit(nameko.cli.main.main())




from isharp.broker_client.remote_proxy import BrokerConnectionPool
import logging
logging.basicConfig()
logger = logging.getLogger()


# create console handler and set level to info
handler = logging.StreamHandler()
handler.setLevel(logging.INFO)
formatter = logging.Formatter("%(levelname)s - %(message)s")
handler.setFormatter(formatter)
logger.addHandler(handler)

logger = logging.getLogger(__name__)


logger.info("hello from logging")


with BrokerConnectionPool() as broker:
    # print(broker.checkout("file://ec2-34-205-159-121.compute-1.amazonaws.com:5672/file_name_1.csv?format=CSV").matrix_header.description)
    # print(broker.checkout("file://ec2-34-205-159-121.compute-1.amazonaws.com:5672/file_name_2.csv?format=CSV").matrix_header.description)
    # print(broker.checkout("file://ec2-34-205-159-121.compute-1.amazonaws.com:5672/subdir_2/subdir_1/file_name_1.csv?format=CSV").matrix_header.description)

    print(broker.list("ec2-34-205-159-121.compute-1.amazonaws.com:5672"))


print("end")





from isharp.core import StorageMethod,MatrixHeader,Revision,AcquireContentReturnValue,MemStyles
from typing import List
import os



class ArcticStorageMethod(StorageMethod):

    def __init__(self,store):
        self.store = store
        super().__init__("arctic",[])

    def _lib_ticker(self,path):
        library, ticker = os.path.split(path)
        library = library.replace("/", "", 1)
        if  library in self.store.list_libraries():
            lib = self.store[library]
            if lib.has_symbol(ticker):
                versioned  = lib.read(ticker)
                return (library,ticker)

            else:
                raise StorageMethod.ResourceException("ticker {} not found".format(ticker))
        else:
            raise StorageMethod.ResourceException("library {} not found".format(library))

        return (library, ticker)

    def acquireContent(self, path, params, version_id=None)->AcquireContentReturnValue:
        self._check_params(params)

        library, ticker = self._lib_ticker(path)
        lib = self.store[library]
        v = None if version_id is None else int(version_id)
        versioned  = lib.read(ticker, v)
        header = MatrixHeader(
        name="",
        revision_id= str(versioned.version),
        storage_method=self.name,
        path= path,
        memory_style=MemStyles.DATA_FRAME,
        description=None
        )


        return AcquireContentReturnValue(content=versioned.data,header=header)




    def storeContent(self, path, params, content,revision_info)->Revision:
        self._check_params(params)
        library, ticker = self._lib_ticker(path)
        lib = self.store[library]
        lib.write(ticker,content )

    def list(self) -> List[MatrixHeader]:
        ret_val = []
        for this_lib_name in self.store.list_libraries():
            library = self.store[this_lib_name]
            for this_symbol in library.list_symbols():
                versions = library.list_versions(this_symbol)
                filtered = [version for version in versions if not version['deleted']]
                max_version = max(map(lambda v: v['version'], filtered))
                ret_val.append(MatrixHeader(name=this_symbol,
                                            description="don't know yet",
                                            storage_method = self.name,
                                            memory_style = MemStyles.DATA_FRAME,
                                            revision_id = str(max_version),
                                            path="{}/{}".format(this_lib_name,this_symbol)))

        return ret_val













from isharp.arctic_broker.broker_impl.arctic_data_broker import  ArcticBroker
from isharp.csv_files.simple_file_broker import SimpleFileBroker
from arctic import Arctic
import yaml
import expandvars


def file_broker_constructor(loader,node):
    root_directory = expandvars.expandvars(node.value[0][1].value)
    print("setting up simple file broker against root dir {}".format(root_directory))
    return SimpleFileBroker(root_directory)

def mongo_broker_constructor (loader,node):
    mongo_location = expandvars.expandvars(node.value[0][1].value)
    print("setting up arctic broker against host {}".format(mongo_location))
    arctic = Arctic(mongo_location)
    return ArcticBroker(arctic)


def set_up_unsafe_loader():
    yaml.add_constructor("!SimpleFileBroker", file_broker_constructor,yaml.UnsafeLoader)
    yaml.add_constructor("!ArcticBroker", mongo_broker_constructor,yaml.UnsafeLoader)











from distutils.core import setup
from setuptools import find_packages

setup(
    name='isharp',
    packages=find_packages(),
    version='1.5',
    description='Algorithmic workbench',
    author='jeremycward',
    author_email='jeremycward@yahoo.co.uk',
    url='https://github.com/jeremycward/isharp-core',
    download_url='https://github.com/jeremycward/isharp-core/tarball/1.00',
    keywords=['string', 'reverse'],
    classifiers=[],
    entry_points={
        'console_scripts': [
            'datahub=isharp.broker_service.datahub_main:main',
        ]
    },
)



import random
import string
from arctic import Arctic
def random_name(name_length):
    return ''.join([random.choice(string.ascii_letters) for n in range(name_length)])

def random_library(arctic_db):
    lib_name = random_name(6)
    while lib_name in arctic_db.list_libraries():
        lib_name = random_name(6)
    return lib_name




